﻿using BinarySearch.State;

namespace BinarySearch.Algorithm;

/// <summary>
/// Класс-Реализатор
/// </summary>
public class BinarySearchAlgorithm
{
    private List<int> sortedArray;
    private int target;
    private int left { get; set; }
    private int right { get; set; }
    private int mid { get; set; }
    private int? targetIndex { get; set; }

    /// <summary>
    /// Конструктор класса-реализатора
    /// </summary>
    /// <param name="array"></param>
    /// <param name="target"></param>
    public BinarySearchAlgorithm(List<int> array, int target)
    {
        targetIndex = null;
        sortedArray = array;
        this.target = target;
        left = 0;
        right = array.Count - 1;
        mid = left + (right - left) / 2;
        SortArray();
    }

    /// <summary>
    /// Сортировка массива
    /// </summary>
    private void SortArray()
    {
        sortedArray.Sort();
    }

    /// <summary>
    /// Совершение шага алгоритма
    /// </summary>
    public void ExecuteStep()
    {
        if (left <= right)
        {
            if (sortedArray[mid] == target)
            {
                targetIndex = mid;
                left = mid + 1;
                right = mid - 1;
                return;
            }
            if (left == right)
            {
                targetIndex = -1;
                left = mid + 1;
                right = mid - 1;
                return;
            }
            if (sortedArray[mid] < target)
            {
                left = mid + 1;
            }
            else
            {
                right = mid - 1;
            }
            mid = left + (right - left) / 2;
        }
        else
        {
            targetIndex = -1;
        }
    }

    /// <summary>
    /// Создание состояние актуального шага
    /// </summary>
    /// <returns></returns>
    public SearchState CreateState()
    {
        return new SearchState(left, right, mid, targetIndex, sortedArray, target);
    }

    /// <summary>
    /// Восстановление шага алгоритма по состоянию
    /// </summary>
    /// <param name="state"></param>
    public void RestoreState(SearchState state)
    {
        left = state.Left;
        right = state.Right;
        mid = state.Mid;
        targetIndex = state.TargetIndex;
        sortedArray = state.Array;
        target = state.Target;
    }

    /// <summary>
    /// Проверка на завершенность алгоритма
    /// </summary>
    /// <returns></returns>
    public bool IsOver()
    {
        return left <= right;
    }
}


