﻿using BinarySearch.State;

namespace BinarySearch.Algorithm;

/// <summary>
/// Класс для визуализации бинарного поиска.
/// </summary>
public class BinarySearchVisualizer
{
    private readonly MainForm _mainForm;
    private readonly List<int> _array;

    /// <summary>
    /// Конструктор класса BinarySearchVisualizer.
    /// </summary>
    /// <param name="mainForm">Главная форма приложения.</param>
    /// <param name="array">Массив, который будет визуализироваться.</param>
    public BinarySearchVisualizer(MainForm mainForm, List<int> array)
    {
        _mainForm = mainForm ?? throw new ArgumentNullException(nameof(mainForm));
        _array = array;
    }

    /// <summary>
    /// Визуализация текущего состояния поиска.
    /// </summary>
    /// <param name="state">Текущее состояние поиска.</param>
    public void Visualize(SearchState state)
    {
        Color color;
        if (state.TargetIndex == null)
        {
            color = Color.Orange;
        } else if (state.TargetIndex >= 0)
        {
            color = Color.Green;
        } else
        {
            color = Color.Red;
        }
        UpdateFlowLayoutPanelByStep(state, color);
    }

    /// <summary>
    /// Обновление панели с элементами.
    /// </summary>
    public void UpdateFlowLayoutPanel()
    {
        _mainForm.flowLayoutPanelArray.Controls.Clear();
        _mainForm.flowLayoutPanelIndex.Controls.Clear();

        if (_array != null)
        {
            for (int i = 0; i < _array.Count; i++)
            {
                TextBox textBox = new()
                {
                    Text = _array[i].ToString(),
                    Width = 40,
                    ReadOnly = false
                };

                TextBox textBoxIndex = new()
                {
                    Text = i.ToString(),
                    Width = 40,
                    ReadOnly = true,
                    Enabled = false
                };

                _mainForm.flowLayoutPanelArray.Controls.Add(textBox);
                _mainForm.flowLayoutPanelIndex.Controls.Add(textBoxIndex);
            }
        }
    }

    /// <summary>
    /// Обновление панели с элементами по шагам
    /// </summary>
    /// <param name="state"></param>
    /// <param name="color"></param>
    private void UpdateFlowLayoutPanelByStep(SearchState state, Color color)
    {
        _mainForm.flowLayoutPanelArray.Controls.Clear();
        _mainForm.flowLayoutPanelIndex.Controls.Clear();
        if (state.Mid < state.Target)
        {
         _mainForm.labelMid.Text = "";
        }
        if (_array != null)
        {
            for (int i = 0; i < _array.Count; i++)
            {
                TextBox textBox = new()
                {
                    Text = _array[i].ToString(),
                    Width = 40,
                    ReadOnly = true
                };

                if (i < state.Left || i > state.Right)
                {
                    textBox.BackColor = Color.Black;
                    textBox.ForeColor = Color.White;
                    textBox.Enabled = false;
                }
                if (i == state.Mid)
                {
                    textBox.BackColor = color;
                    textBox.ForeColor = Color.White;
                    textBox.Enabled = true;
                }

                TextBox textBoxIndex = new()
                {
                    Text = i.ToString(),
                    Width = 40,
                    ReadOnly = true,
                    Enabled = false
                };

                _mainForm.flowLayoutPanelArray.Controls.Add(textBox);
                _mainForm.flowLayoutPanelIndex.Controls.Add(textBoxIndex);
            }
        }
    }
}
