﻿using BinarySearch.State;

namespace BinarySearch.Algorithm;

/// <summary>
/// Класс, управляющий выполнением бинарного поиска.
/// </summary>
public class Manager
{
    private readonly StateStorage _storage;

    /// <summary>
    /// Конструктор класса Manager
    /// </summary>
    /// <param name="storage">Хранилище состояний.</param>
    public Manager(StateStorage storage)
    {
        _storage = storage;
    }

    /// <summary>
    /// Выполнение бинарного поиска.
    /// </summary>
    /// <param name="parameters">Параметры поиска.</param>
    public void ExecuteBinarySearch(SearchParameters parameters)
    {
        if (parameters == null)
        {
            throw new ArgumentNullException(nameof(parameters));
        }
        BinarySearchAlgorithm algorithm = new(parameters.Array, parameters.Target);
        SearchState? state;
        _storage.Clear();

        while (algorithm.IsOver())
        {
            state = algorithm.CreateState();
            _storage.AddState(state);
            algorithm.ExecuteStep();
        }
        state = algorithm.CreateState();
        _storage.AddState(state);
    }
}
