﻿using BinarySearch.Algorithm;
using BinarySearch.Forms;
namespace BinarySearch;

/// <summary>
/// Форма для инициализации данных перед выполнением поиска.
/// </summary>
public partial class InitDataForm : Form
{
    /// <summary>
    /// Делегат для передачи параметров поиска.
    /// </summary>
    /// <param name="searchParameters">Параметры поиска.</param>
    public delegate void InitParametersDelegate(SearchParameters searchParameters);

    /// <summary>
    /// Событие, срабатывающее при инициализации параметров поиска.
    /// </summary>
    public event InitParametersDelegate? InitParametersEvent;

    private List<int> _arrayInit;

    /// <summary>
    /// Инициализирует новый экземпляр формы InitDataForm.
    /// </summary>
    public InitDataForm()
    {
        _arrayInit = new List<int>();
        InitializeComponent();
    }

    /// <summary>
    /// Обновляет элементы списка на форме.
    /// </summary>
    private void UpdateListBox()
    {
        listBoxArray.Items.Clear();
        foreach (int number in _arrayInit)
        {
            listBoxArray.Items.Add(number);
        }
    }

    /// <summary>
    /// Обработчик события нажатия кнопки "Добавить число".
    /// Открывает диалоговое окно для добавления числа.
    /// </summary>
    private void buttonAdd_Click(object sender, EventArgs e)
    {
        AddForm addForm = new();
        addForm.AddNumberEvent += AddNumberEventHandler;
        addForm.ShowDialog();
    }

    /// <summary>
    /// Обработчик события добавления числа из диалогового окна.
    /// Добавляет число в список и обновляет элементы списка на форме.
    /// </summary>
    /// <param name="number">Добавленное число.</param>
    private void AddNumberEventHandler(int number)
    {
        _arrayInit.Add(number);
        UpdateListBox();
    }

    /// <summary>
    /// Обработчик события нажатия кнопки "Сбросить".
    /// Очищает список и обновляет элементы списка на форме.
    /// </summary>
    private void buttonReset_Click(object sender, EventArgs e)
    {
        _arrayInit.Clear();
        UpdateListBox();
        numericUpDownNumber.Value = 0;
    }

    /// <summary>
    /// Обработчик события нажатия кнопки "Сохранить".
    /// Формирует объект параметров поиска и вызывает событие инициализации параметров поиска.
    /// </summary>
    private void buttonSave_Click(object sender, EventArgs e)
    {
        SearchParameters parameters = new SearchParameters(_arrayInit, (int)numericUpDownNumber.Value);
        InitParametersEvent?.Invoke(parameters);

        this.Close();
    }
}

